// Contains class definitions
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Artistic License, included as the file
// "LICENSE" in the source code archive.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// You should have received a copy of the file "LICENSE", containing
// the License John Weiss originally placed this program under.
#include "AudDB.h"


// ***************
// Here, we define the actual functions



//||||||||||||||||||||||||||||||||||||||||||||||||
//
//  Class AudDB
//
//||||||||||||||||||||||||||||||||||||||||||||||||


/* 
 * Constructor for the AudDB object.
 * Sets all poiners and counters to 0.
 *
 */
AudDB::AudDB(void)
{
   db_name = NULL;
   magiclist = NULL;
   len_magic = 0;
}


/* 
 * Overloaded Constructor.  Difference:  \ref{db_name} is set to a
 * duplicate of the argument {\em name}.
 * 
 */
AudDB::AudDB(char *name)
{
   magiclist = NULL;
   len_magic = 0;
   setName(name);
}


/*
 * Destructor for the AudDB object.
 * 
 */
AudDB::~AudDB(void)
{
   if(magiclist != NULL)
	 delete [] magiclist;

   if(db_name != NULL)
	 delete [] db_name;
}

/*
 * Sets \Ref{db_name} to a copy of {\em name}.
 */
void AudDB::setName(char *name)
{
   db_name = new char[strlen(name)+1];
   strcpy(db_name,name);
}   


/*
 * Adds the catalog {\em ct} to the database, overwriting any existing
 * entry for {\em ct}.
 * 
 */
void AudDB::add(Catalog &ct)
{
   unsigned long *oldml;
   int i,oldlen;

   oldlen = len_magic;
   write(ct,len_magic);

   if(oldlen < len_magic)
	 {
	 oldml = magiclist;
	 magiclist = new unsigned long[len_magic];
	 for(i=0; i<oldlen; i++)
	    magiclist[i] = oldml[i];
	 magiclist[oldlen] = ct.magic();
	 if(oldml != NULL)
	    delete [] oldml;
	 }
}


/*
 * Searches for the catalog {\em ct} in the database.
 * @return a location index or, -1 if {\em ct} is not in the database.
 * 
 */
int AudDB::find(Catalog& ct)
{
   unsigned long mg;
   int i,locn;

   mg = ct.magic();
   locn = -1;
   i=0;
#ifdef DEBUG_ON
   cerr << "%dbg%\tAudDB::find(): Magic number=" << hex
	   << mg << dec << endl;
#endif
   while( (i < len_magic) && (locn == -1) )
	 {
#ifdef DEBUG_ON
	 cerr << "\t\tmagic[" << i << "]=" << hex
		 << magiclist[i] << dec << endl;
#endif
	 if( magiclist[i] == mg )
	    locn = i;
	 i++;
	 }
#ifdef DEBUG_ON
	 cerr << "\tlocation=" << locn << endl;
#endif
   return(locn);
}


// Reads a catalog entry. Overloaded version that only loads the
// catalog if ct.magic() matches an entry.
//
bool AudDB::read(Catalog& ct)
{
   int idx2;

   idx2 = find(ct);
   return read_i( ct, idx2 );
}





//||||||||||||||||||||||||||||||||
//
// End
//
