//-*-C++-*- Header file for the TiKanDB Class
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Artistic License, included as the file
// "LICENSE" in the source code archive.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// You should have received a copy of the file "LICENSE", containing
// the License John Weiss originally placed this program under.
//
// This file is unreadable, because the comment-docifier is
// brainedead.  Try a "make srcdocs" to generate the
// HTML-documentation.
//
#ifndef _TiKanDB_h_
#define _TiKanDB_h_
#include "AudDB.h"


//
#define LINEBUF 1024  //1k lines max.
//
#define DEFPATH pathlst[0]
/**
  Upper limit on the number of paths \Ref{TiKanDB} will hold and
  process.  Currently set to 32.
  @memo \Ref{TiKanDB} path limit.
  */
#define NPATH_MAX 32


/**
  {\bf TiKanDB} is a {\bf AudDB} implemented to work with
  xmcd-style databases.

  @memo xmcd-style audio database.
 */
class TiKanDB : public AudDB
{
   public:
	 /** Class Constructor
	   @memo
	   */
	 TiKanDB(void);

	 /** Overloaded Constructor.  Sets \Ref{db_name} to
	   a duplicate of the argument {\em fpath}, which should be a ':'
	  -separated list of search paths.  You can only specify
	  \Ref{NPATH_MAX} paths.
	   @memo
	   */
	 TiKanDB(char *fpath);

	 /** Class Destructor.
	   @memo
	   */
	 ~TiKanDB(void);

	 /** {\tt [virtual]}

	   Reimplements the \Ref{AudDB} version. 
	   Recursivesly scans the ':'-separated list of paths specified
	   in \Ref{db_name}.  Symbolic links should point to directories
	   only, not files.
	   Because xmcd-style databases reside in individual files under
	   a directory tree, {\bf open()} stores the full filename of
	   each database entry.
	   @memo
	   */
	 void open(void);

   protected:
	 /**
	   An array containing the full filename of each database entry.
	   Has the same length as \Ref{magiclist}.
	   @memo
	   */
	 char **pathpos;
	 /**
	   The list of paths specified in \Ref{db_name}, divided into an
	   array.
	   @memo
	   */
	 char **pathlst;
	 /**
	   The number of paths specified in \Ref{db_name} and
	   subsequently stored in \Ref{pathlst}.
	   @memo
	   */
	 int npaths;

	 /** {\tt [virtual]}

	   Reimplements the \Ref{AudDB} version. If {\em ct} is
	   already in the database, it increments {\em oldlen}.
	   @memo
	   */
	 void write(Catalog& ct, int& oldlen);

	 /** {\tt [virtual]}

	   Reimplements the \Ref{AudDB} version. Opens an input stream
	   for the file at \Ref{pathpos}[{\em idx}], then calls
	   \Ref{readEntry}.
	   @memo
	   */
	 bool read_i(Catalog& ct, int idx);

	 /** {\tt [virtual]}

	   Reimplements the \Ref{AudDB} version.
	   @memo
	   */
	 bool readEntry(ifstream& instr, Catalog& ct);

	 /** {\tt [virtual]}

	   Reimplements the \Ref{AudDB} version.
	   @memo
	   */
	 bool writeEntry(ofstream& outstr, Catalog& ct);

	 //
   private:
	 int tmplen;
};

#endif // _TiKanDB_h_
//||||||||||||||||||||||||||||||||
//
// End
// 
